package cn.gygxzc.cloud.config.service.impl

import cn.gygxzc.cloud.config.SEP_DOT
import cn.gygxzc.cloud.config.dao.ILabelDao
import cn.gygxzc.cloud.config.model.PropertyLabel
import cn.gygxzc.cloud.config.service.ILabelService
import cn.tonyandmoney.tuonq.core.req.PageReq
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper
import com.baomidou.mybatisplus.core.metadata.IPage
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import reactor.core.publisher.Mono

@Service
class LabelService : ILabelService {

    @Autowired
    private lateinit var labelDao: ILabelDao

    /**
     * 根据给定的条件分页查询数据
     */
    override fun findAll(req: PageReq, keywords: String?): Mono<IPage<PropertyLabel>> {
        return Mono.create {
            val page = req.page<PropertyLabel>()
            labelDao.selectPage(page, QueryWrapper<PropertyLabel>().apply {
                if (!keywords.isNullOrBlank()) {
                    likeRight("path", keywords)
                    or().like("label", keywords)
                }
            })
            it.success(page)
        }
    }

    override fun addLabel(label: PropertyLabel): Mono<Void> {
        return Mono.create {
            val pId = label.pId ?: 0
            if (pId > 0) {
                val pLabel = labelDao.selectOne(QueryWrapper<PropertyLabel>().apply {
                    eq("id", pId)
                    select("path", "id")
                })
                label.path = "${pLabel.path}$SEP_DOT${label.label}"
            } else {
                label.path = label.label
            }

            labelDao.insert(label)
            it.success()
        }
    }

    override fun editLabel(label: PropertyLabel): Mono<Void> {
        return Mono.create {
            label.path = null
            label.pId = null
            label.label = null
            labelDao.updateById(label)
            it.success()
        }
    }

    override fun delLabel(id: Int): Mono<Void> {
        return Mono.create {
            val label = labelDao.selectById(id)
            if (label == null) {
                it.success()
                return@create
            }
            val count = labelDao.selectCount(QueryWrapper<PropertyLabel>().apply {
                likeRight("path", label.path)
            })
            if (count > 1) {
                it.error(RuntimeException("该标签有子级，请先删除。"))
            } else {
                labelDao.deleteById(id)
                it.success()
            }
        }
    }

}